#include "Engine/Entity/Animation.h"
#include "jlog/Logger.hpp"

float Animation::GetMsBetweenFrames() const {
    return ms_between_frames;
}

void Animation::SetMsBetweenFrames(float new_ms_between_frames) {
    ms_between_frames = new_ms_between_frames;
}

std::string Animation::GetName() const {
    return name;
}

const JGL::Texture* Animation::GetTexture(float animation_time) const {
    if (textures.empty())
        jlog::Fatal("Getting an animation frame but the animation is empty?");

    animation_time = std::max(0.0f, std::min(animation_time, 1.0f));
    float animation_ms = ms_between_frames * textures.size();
    float animation_length_ms = animation_time * animation_ms;
    auto frame_index = (size_t) std::floor(animation_length_ms / ms_between_frames);
    frame_index = std::min(frame_index, textures.size() - 1);

    return &textures[frame_index];
}
