#include <J3ML/J3ML.hpp>
#include <Engine/Entity/Entity.h>
#include <Engine/Globals.h>

using namespace J3ML;
void Engine::Entity::MoveX(float speed) {
    position.x = position.x + (speed * Globals::Window->GetDeltaTime());
}

void Engine::Entity::MoveY(float speed) {
    position.y = position.y + (speed * Globals::DeltaTime());
}

void Engine::Entity::Move(float angle_rad, float speed) {
    position.x = position.x + (speed * Globals::DeltaTime()) * Math::Cos(angle_rad);
    position.y = position.y + (speed * Globals::DeltaTime()) * Math::Sin(angle_rad);
}

void Engine::Entity::MoveForward(float speed) {
    Move(face_angle, speed);
}

void Engine::Entity::MoveBackward(float speed) {
    speed = -speed;
    position.x = position.x + (speed * Globals::DeltaTime()) * Math::Cos(face_angle);
    position.y = position.y + (speed * Globals::DeltaTime()) * Math::Sin(face_angle);
}

void Engine::Entity::MoveLeft(float speed) {
    position.x = position.x + (speed * Globals::DeltaTime()) * -Math::Sin(face_angle);
    position.y = position.y + (speed * Globals::DeltaTime()) * Math::Cos(face_angle);
}

void Engine::Entity::MoveRight(float speed) {
    position.x = position.x + (speed * Globals::DeltaTime()) * Math::Sin(face_angle);
    position.y = position.y + (speed * Globals::DeltaTime()) * -Math::Cos(face_angle);
}

void Engine::Entity::SetRotation(float new_face_angle) {
    face_angle = new_face_angle;

    if (face_angle < 0)
        face_angle = fmod(face_angle + Math::Pi * 2, Math::Pi * 2);
    else if (face_angle >= 2 * M_PI)
        face_angle = fmod(face_angle, Math::Pi * 2);
}

void Engine::Entity::Rotate(float speed) {
    SetRotation(face_angle + speed * Globals::DeltaTime());
}

float Engine::Entity::GetRotation() const {
    return face_angle;
}

Vector2 Engine::Entity::GetPosition() const {
    return position;
}

bool Engine::Entity::AppendChild(Entity* entity) {
    bool success = false;
    if (!Globals::CurrentScene->EntityListContains(entity)) {
        entity->parent = this;
        children.push_back(entity);
        success = true;
    }
    return success;
}

Engine::Entity::~Entity() {
    for (auto* e : children)
        delete e;
}

void Engine::Entity::UpdateChildren() {
    for (auto& e : children) {
        e->Update();
        if (!e->children.empty())
            e->UpdateChildren();
    }
}

void Engine::Entity::DestroyChild(Entity* entity) {
    auto it = std::find(children.begin(), children.end(), entity);
    if (it != children.end())
        delete *it, children.erase(it);
}

void Engine::Entity::RemoveChild(Entity* entity) {
    auto it = std::find(children.begin(), children.end(), entity);
    if (it != children.end())
        children.erase(it);
}

long Engine::Entity::GetCreationTime() const {
    return creation_time;
}

long Engine::Entity::GetAge() const {
    return std::chrono::duration_cast<std::chrono::microseconds>(std::chrono::high_resolution_clock::now().time_since_epoch()).count() - creation_time;
}

Engine::Entity* Engine::Entity::GetParent() const {
    return parent;
}

std::vector<Engine::Entity*> Engine::Entity::GetChildren() const {
    return children;
}
