#include <Engine/Level/Fixed.h>

bool Engine::Fixed::Collidable() const {
    return collidable;
}

Vector2i Engine::Fixed::GetPosition() const {
    return position;
}

AABB2D Engine::Fixed::GetBounds() const {
    auto maximum = Vector2(position.x + texture->GetDimensions().x, position.y + texture->GetDimensions().y);
    return { Vector2(position), maximum };
}

bool Engine::Fixed::Enabled() const {
    return enabled;
}

void Engine::Fixed::Enable() {
    enabled = true;
}

void Engine::Fixed::Disable() {
    enabled = false;
}

Engine::Fixed::~Fixed() {
    delete texture;
}

void Engine::Fixed::GenerateCollision() {
    if (!Collidable() || !Enabled() || !texture)
        return;

    std::vector<Vector2i> result{};
    auto pixel_data = texture->GetPixelData();

    for (int y = 0; y < texture->GetDimensions().y; y++)
        for (int x = 0; x < texture->GetDimensions().x; x++)
            if (pixel_data[y * texture->GetDimensions().x + x].A() != 0)
                result.emplace_back((int) x + position.x, (int) y + position.y);
    collision = result;
}

std::vector<Vector2i> Engine::Fixed::GetCollision() {
    return collision;
}

const Texture* Engine::Fixed::GetTexture() {
    return texture;
}
